/* PLOP sample application for dumping PDF information with pCOS functions
 *
 * $Id: dumper.c,v 1.21 2016/04/27 15:17:12 tm Exp $
 */

#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "ploplib.h"

int
main(int argc, char **argv)
{
    PLOP	*p;

    /* This is where input files live. Adjust as necessary. */
    const char *searchpath = "../data";

    if (argc != 2)
    {
        fprintf(stderr, "usage: dumper <filename>\n");
        return(2);
    }

    if ((p = PLOP_new()) == (PLOP *) 0)         /* out of memory */
    {
	fprintf(stderr, "dumper: out of memory\n");
	return 99;
    }

    PLOP_TRY(p)
    {
	int		count, pcosmode, recipientcount, plainmetadata;
	int		objtype, i, doc;
	const char	*docoptlist = "requiredmode=minimum";
	const char	*globaloptlist = "";
	char optlist[1024];

	PLOP_set_option(p, globaloptlist);

	sprintf(optlist, "searchpath={%s}", searchpath);
	PLOP_set_option(p, optlist);

	if ((doc = PLOP_open_document(p, argv[1], 0, docoptlist)) == -1)
	{
	    printf("ERROR: %s\n", PLOP_get_errmsg(p));
	    PLOP_delete(p);
	    return 98;
	}

	/* --------- general information (always available) */

	pcosmode = (int) PLOP_pcos_get_number(p, doc, "pcosmode");

	printf("       File name: %s\n", PLOP_pcos_get_string(p, doc, "filename"));

	printf("     PDF version: %s\n",
	    PLOP_pcos_get_string(p, doc, "pdfversionstring"));

	printf("      Encryption: %s\n",
	    PLOP_pcos_get_string(p, doc, "encrypt/description"));

	recipientcount =
		(int) PLOP_pcos_get_number(p, doc, "length:encrypt/recipients");

	if (recipientcount > 0)
	{
	    printf("Recipient groups: %d\n", recipientcount);
	    printf("      Digital ID: ");

	    if (pcosmode == 2)
		printf("supplied digital ID has master permissions\n");
	    else if (pcosmode == 1)
	        printf("supplied digital ID doesn't have master permissions\n");
	    else
	        printf("no suitable digital ID supplied\n");
	}

	printf("       Master pw: %s\n",
	    PLOP_pcos_get_number(p, doc, "encrypt/master") ? "yes" : "no");

	printf("         User pw: %s\n",
	    PLOP_pcos_get_number(p, doc, "encrypt/user") ? "yes" : "no");

	printf("    Text copying: %s\n",
	    PLOP_pcos_get_number(p, doc, "encrypt/nocopy") ? "no" : "yes");

	printf("      Linearized: %s\n",
	    PLOP_pcos_get_number(p, doc, "linearized") ? "yes" : "no");

	if (pcosmode == 0)
	{
	    printf("    Minimum mode: no more information available\n\n");
            PLOP_close_document(p, doc, "");
	    PLOP_delete(p);
	    return 0;
	}

	/* --------- more details (requires at least user password) */

	printf("    PDF/X status: %s\n", PLOP_pcos_get_string(p, doc, "pdfx"));

	printf("    PDF/A status: %s\n", PLOP_pcos_get_string(p, doc, "pdfa"));

	printf("   PDF/UA status: %s\n", PLOP_pcos_get_string(p, doc, "pdfua"));

	printf("   PDF/VT status: %s\n", PLOP_pcos_get_string(p, doc, "pdfvt"));

	printf("        XFA data: %s\n",
                (int) PLOP_pcos_get_number(p, doc, "type:/Root/AcroForm/XFA")
	                                    != pcos_ot_null
	                            ? "yes" : "no");
	printf("      Tagged PDF: %s\n\n",
	    PLOP_pcos_get_number(p, doc, "tagged") ? "yes" : "no");

        /* --------- analyze digital signatures */

	count = (int) PLOP_pcos_get_number(p, doc, "length:signaturefields");

        for (i=0; i < count; i++)
        {
            const char *sigtype;

            printf("   Sig. field #%d: '%s',", i+1,
                PLOP_pcos_get_string(p, doc,
                    "signaturefields[%d]/fullname", i));

            printf(" %s", (PLOP_pcos_get_number(p, doc,
                "signaturefields[%d]/visible", i) ? "visible" : "invisible"));

            sigtype = PLOP_pcos_get_string(p, doc,
                "signaturefields[%d]/sigtype", i);

            if (!strcmp(sigtype, "none"))
            {
                printf(", unsigned\n");
                continue;
            }

            printf(" %s signature", sigtype);

            if (!strcmp(sigtype, "certification"))
            {
                printf(" (%s allowed)", PLOP_pcos_get_string(p, doc,
                        "signaturefields[%d]/permissions", i));
            }

            if (PLOP_pcos_get_number(p, doc, "signaturefields[%d]/cades", i))
                printf(", CAdES");

            printf("\n");
        }
        if (count > 0)
        {
            printf("No. of revisions: %d\n\n",
                    (int) PLOP_pcos_get_number(p, doc, "revisions"));
        }

        printf("  Reader-enabled: %s\n",
            PLOP_pcos_get_string(p, doc, "usagerights"));

	printf("    No. of pages: %d\n",
	    (int) PLOP_pcos_get_number(p, doc, "length:pages"));

	printf("     Page 1 size: width=%g, height=%g\n",
	    PLOP_pcos_get_number(p, doc, "pages[%d]/width", 0),
	    PLOP_pcos_get_number(p, doc, "pages[%d]/height", 0));

	count = (int) PLOP_pcos_get_number(p, doc, "length:fonts");
	printf("    No. of fonts: %d\n", count);

	for (i=0; i < count; i++)
	{
	    if (PLOP_pcos_get_number(p, doc, "fonts[%d]/embedded", i))
		printf("embedded ");
	    else
		printf("unembedded ");

	    printf("%s font ",
		PLOP_pcos_get_string(p, doc, "fonts[%d]/type", i));
	    printf("%s\n",
		PLOP_pcos_get_string(p, doc, "fonts[%d]/name", i));
	}

	printf("\n");

	plainmetadata = (int)
	    PLOP_pcos_get_number(p, doc, "encrypt/plainmetadata");

	if (pcosmode == 1 && !plainmetadata &&
	    ((int) PLOP_pcos_get_number(p, doc, "encrypt/nocopy")))
	{
	    printf("Restricted mode: no more information available\n\n");
            PLOP_close_document(p, doc, "");
	    PLOP_delete(p);
	    return 0;
	}

	/* --------- document info keys and XMP metadata (requires master pw
	 * or plaintext metadata)
	 */

	count = (int) PLOP_pcos_get_number(p, doc, "length:/Info");

	for (i=0; i < count; i++)
	{
	    objtype = (int) PLOP_pcos_get_number(p, doc, "type:/Info[%d]", i);
	    printf("%16s: ", PLOP_pcos_get_string(p, doc, "/Info[%d].key", i));

	    /* Info entries can be stored as string or name objects */
	    if (objtype == pcos_ot_string || objtype == pcos_ot_name)
	    {
		printf("'%s'\n",
			PLOP_pcos_get_string(p, doc, "/Info[%d]", i));
	    }
	    else
	    {
		printf("(%s object)\n",
		    PLOP_pcos_get_string(p, doc, "type:/Info[%d]", i));
	    }
	}

	printf("\n   XMP meta data: ");

	objtype = (int)PLOP_pcos_get_number(p, doc, "type:/Root/Metadata");
	if (objtype == pcos_ot_stream)
	{
	    const char *contents;
	    int len;

	    contents = (const char *)
		PLOP_pcos_get_stream(p, doc, &len, "", "/Root/Metadata");
	    printf("%d bytes ", len);

	    /* This demonstrates Unicode conversion. The UTF-8 XMP data
	     * is converted to UTF-16.
	     */
	    PLOP_convert_to_unicode(p, "utf8", contents, 0, &len,
		"outputformat=utf16");
	    printf("(%d Unicode characters)\n\n", len/2);
	}
	else
	{
	    printf("not present\n\n");
	}

        PLOP_close_document(p, doc, "");
    }
    PLOP_CATCH(p)
    {
        printf("pCOS exception occurred in dumper:\n");
        printf("[%d] %s: %s\n",
            PLOP_get_errnum(p), PLOP_get_apiname(p),
	    PLOP_get_errmsg(p));
    }
    PLOP_delete(p);

    return 0;
}
